Type: Package
Package: QuadratiK
Title: Collection of Methods Constructed using Kernel-Based Quadratic
        Distances
Version: 1.2.0
Authors@R: c(
    person("Giovanni", "Saraceno", ,"giovanni.saraceno@unipd.it", role = c("aut", "cre"),
           comment = "ORCID 000-0002-1753-2367"),
    person("Marianthi", "Markatou", role = "aut"),
    person("Raktim", "Mukhopadhyay", role = "aut"),
    person("Mojgan", "Golzy", role = c("aut")),
    person("Hingee", "Kassel", role = "rev"),
    person("Emi", "Tanaka", role = "rev")
  )
Maintainer: Giovanni Saraceno <giovanni.saraceno@unipd.it>
Description: It includes test for multivariate normality, test for uniformity on the d-dimensional 
    Sphere, non-parametric two- and k-sample tests, random generation of points from the Poisson 
    kernel-based density and clustering algorithm for spherical data. For more information see
    Saraceno G., Markatou M., Mukhopadhyay R. and Golzy M. (2024)
    <doi:10.48550/arXiv.2402.02290>
    Markatou, M. and Saraceno, G. (2024) <doi:10.48550/arXiv.2407.16374>, 
    Ding, Y., Markatou, M. and Saraceno, G. (2023) <doi:10.5705/ss.202022.0347>, 
    and Golzy, M. and Markatou, M. (2020) <doi:10.1080/10618600.2020.1740713>.
License: GPL (>= 3)
URL: https://CRAN.R-project.org/package=QuadratiK,
        https://github.com/ropensci/QuadratiK/,
        https://docs.ropensci.org/QuadratiK/
BugReports: https://github.com/ropensci/QuadratiK/issues
Depends: R (>= 3.5.0)
Imports: parallel, doParallel, foreach, ggplot2, ggpubr, methods,
        moments, mvtnorm, Rcpp, RcppEigen, rlecuyer, sn, stats, rrcov,
        scatterplot3d
Suggests: knitr, rmarkdown, roxygen2, testthat (>= 3.0.0), rgl,
        sphunif, circular, cluster, clusterRepro, mclust, Tinflex,
        movMF
LinkingTo: Rcpp, RcppEigen
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Language: en-US
NeedsCompilation: yes
Packaged: 2026-05-03 14:11:42 UTC; sarac
Author: Giovanni Saraceno [aut, cre] (ORCID 000-0002-1753-2367),
  Marianthi Markatou [aut],
  Raktim Mukhopadhyay [aut],
  Mojgan Golzy [aut],
  Hingee Kassel [rev],
  Emi Tanaka [rev]
Repository: CRAN
Date/Publication: 2026-05-03 16:10:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-03 17:57:49 UTC; unix
Archs: QuadratiK.so.dSYM
