# Generated by roxygen2: do not edit by hand

export(dpkb)
export(kb.test)
export(pk.test)
export(pkbc)
export(pkbc_validation)
export(rpkb)
export(sample_hypersphere)
export(select_h)
export(stats_clusters)
exportMethods(kb.test)
exportMethods(pk.test)
exportMethods(pkbc)
exportMethods(plot)
exportMethods(predict)
exportMethods(show)
exportMethods(stats_clusters)
exportMethods(summary)
import(Rcpp)
import(RcppEigen)
import(ggplot2)
import(rlecuyer)
importFrom(Rcpp,sourceCpp)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_brewer)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggpubr,ggarrange)
importFrom(grDevices,rainbow)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(methods,new)
importFrom(moments,skewness)
importFrom(mvtnorm,rmvnorm)
importFrom(parallel,clusterExport)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(rrcov,PcaLocantore)
importFrom(scatterplot3d,scatterplot3d)
importFrom(sn,rmsn)
importFrom(stats,IQR)
importFrom(stats,aggregate)
importFrom(stats,cov)
importFrom(stats,dist)
importFrom(stats,median)
importFrom(stats,power)
importFrom(stats,qchisq)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(utils,install.packages)
useDynLib(QuadratiK)
